<?php
/*
Plugin Name: Hierarchical Navigation
Plugin URI: http://www.christianschenk.org/projects/wordpress-hierarchical-navigation-plugin/
Description: Helps you to embed a hierarchical navigation menu and breadcrumbs into your site.
Version: 1.1
Author: Christian Schenk
Author URI: http://www.christianschenk.org/
*/

#
# WordPress Hierarchical Navigation plugin
# Copyright (C) 2009-2013 Christian Schenk
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#


/**
 * Returns the posts/pages for a given parent page.
 */
function hiernav_get_postpage_ids($parent = 0, $exclude = array(), $post_type = 'page', $order = 'menu_order') {
	global $wpdb;
	$result = $wpdb->get_results('SELECT ID FROM '.$wpdb->posts.' WHERE post_type = "'.$post_type.'" AND post_parent = '.$parent.' ORDER BY '.$order);

	$ids = array();
	foreach ($result as $key => $value) {
		$id = $result[$key]->ID;
		if (in_array($id, $exclude)) continue;
		$ids[] = $id;
	}

	return $ids;
}


/**
 * Returns a CSV for an array of IDs.
 */
function hiernav_get_csv_ids($ids) {
	$ids_csv = '';
	foreach ($ids as $id)
		$ids_csv .= $id.',';
	return substr($ids_csv, 0, strlen($ids_csv) - 1);
}


/**
 * Returns the hierarchical menu.
 */
function hiernav_the_hierarchical_menu($start_id = 0, $depth = 3, $exclude = array(), $title_li = '<h2></h2>') {
	global $post;

	# Preparation
	if (!is_array($exclude))
		$exclude = explode(',', $exclude);

	$ids = hiernav_get_postpage_ids($start_id, $exclude);
	$ids_csv = hiernav_get_csv_ids($ids);

	# Search for children
	$child_ids = array();
	$ancestors = (!empty($post->ancestors)) ? $post->ancestors : array();
	foreach (array_merge($ancestors, array($post->ID)) as $id)
		foreach (hiernav_get_postpage_ids($id, $exclude) as $child_id)
			$child_ids[] = $child_id;
	$ids_csv .= ','.hiernav_get_csv_ids($child_ids);

	echo '<ul>';
	wp_list_pages('depth='.$depth.'&title_li='.$title_li.'&include='.$ids_csv);
	echo '</ul>';
}


/**
 * Almost copied from here:
 * http://www.itsananderson.com/2008/12/wordpress-breadcrumbs/
 */
function hiernav_the_breadcrumbs($separator = ' &raquo; ', $top_title = '') {
	global $post;

	$trail = array($post);
	$parent = $post;
	$show_on_front = get_option('show_on_front');
	$page_on_front = get_option('page_on_front');
	while ($parent->post_parent && !($parent->ID == $page_on_front && 'page') == $show_on_front) {
		$parent = get_post($parent->post_parent);
		array_unshift($trail, $parent);
	}
	if ('posts' == $show_on_front) {
		$top_title = (empty($top_title)) ? __('Welcome', 'hiernav') : $top_title;
		echo '<a href="' . get_bloginfo('home') . '">'.$top_title.'</a>';
	} else {
		$home_page = get_post($page_on_front);
		echo '<a href="' . get_bloginfo('home') . '">'.$home_page->post_title.'</a>';

		if($trail[0]->ID == $page_on_front)
			array_shift($trail);
	}

	foreach ($trail as $page) {
		if ($page->ID == $post->ID)
			echo $separator.$page->post_title;
		else
			echo $separator.'<a href="' . get_page_link($page->ID) . '">'.$page->post_title.'</a>';
	}
}


/**
 * Hierarchical navigation menu widget
 */
class HierachicalNavigation_Widget extends WP_Widget {
	function HierachicalNavigation_Widget() {
		$widget_ops = array('classname' => 'widget_hiernav', 'description' => __('Hierarchical navigation menu', 'hiernav'));
		$this->WP_Widget('hiernav', __('Hierarchical Navigation', 'hiernav'), $widget_ops);
	}

	function get_my_widget_options() {
		return array('title' => NULL,
		             'start_id' => 0,
		             'depth' => 3,
		             'exclude' => NULL,
		             'title_li' => '<h2></h2>');
	}

	function get_my_widget_option_name($key) {
		$names = array('title' => 'Title',
		               'start_id' => 'Start ID<br/><span style="font-size:smaller">The navigation hierarchy starts with this post\'s or page\'s ID. Use 0 for root.</span>',
		               'depth' => 'Depth<br/><span style="font-size:smaller">The navigation is this deep.</span>',
		               'exclude' => 'Exclude IDs<br/><span style="font-size:smaller">Comma separated list of IDs.</span>',
		               'title_li' => 'Title Markup<br/><span style="font-size:smaller">Markup used for the list\'s title.</span>');
		return $names[$key];
	}

	function widget($args, $instance) {
		extract($args, EXTR_SKIP);

		echo $before_widget;
		if (strlen(trim($instance['title'])) != 0) {
			echo $before_title;
			echo $instance['title'];
			echo $after_title;
		}
		hiernav_the_hierarchical_menu($instance['start_id'], $instance['depth'], $instance['exclude'], $instance['title_li']);
		echo $after_widget;
	}

	function update($new_instance, $old_instance) {
		$instance = $old_instance;
		foreach ($this->get_my_widget_options() as $key => $value) {
			$instance[$key] = $new_instance[$key];
		}
		return $instance;
	}

	function form($instance) {
		$instance = wp_parse_args( (array) $instance, $this->get_my_widget_options());

		foreach ($this->get_my_widget_options() as $key => $value) { 
			$realValue = attribute_escape($instance[$key]);
			if (!isset($realValue) and $value != $realValue) $realValue = $value;
		?>
		<p><label for="<?php echo $this->get_field_id($key); ?>"><br/><?php echo $this->get_my_widget_option_name($key); ?><br/>
			<input class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" type="text" value="<?php echo $realValue; ?>" />
		</label></p>
		<?php }
	}
}
if (function_exists('add_action'))
	add_action('widgets_init', create_function('', 'return register_widget("HierachicalNavigation_Widget");'));


/**
 * Breadcrumbs widget
 */
class HierNavBreadcrumbs_Widget extends WP_Widget {
	function HierNavBreadcrumbs_Widget() {
		$widget_ops = array('classname' => 'widget_hiernavbc', 'description' => __('Breadcrumbs guide you back home...', 'hiernav'));
		$this->WP_Widget('hiernavbc', __('Breadcrumbs', 'hiernav'), $widget_ops);
	}

	function get_my_widget_options() {
		return array('title' => NULL, 'separator' => ' &raquo; ', 'top_title' => __('Welcome', 'hiernav'));
	}

	function widget($args, $instance) {
		extract($args, EXTR_SKIP);

		echo $before_widget;
		if (strlen(trim($instance['title'])) != 0) {
			echo $before_title;
			echo $instance['title'];
			echo $after_title;
		}
		hiernav_the_breadcrumbs($instance['separator'] ,$instance['top_title']);
		echo $after_widget;
	}

	function update($new_instance, $old_instance) {
		$instance = $old_instance;
		foreach ($this->get_my_widget_options() as $key => $value) {
			$instance[$key] = $new_instance[$key];
		}
		return $instance;
	}

	function form($instance) {
		$instance = wp_parse_args( (array) $instance, $this->get_my_widget_options());

		foreach ($this->get_my_widget_options() as $key => $value) { 
			$realValue = attribute_escape($instance[$key]);
			if (!isset($realValue) and $value != $realValue) $realValue = $value;
		?>
		<p><label for="<?php echo $this->get_field_id($key); ?>"><br/><?php echo $key; ?>:<br/>
			<input class="widefat" id="<?php echo $this->get_field_id($key); ?>" name="<?php echo $this->get_field_name($key); ?>" type="text" value="<?php echo $realValue; ?>" />
		</label></p>
		<?php }
	}
}
if (function_exists('add_action'))
	add_action('widgets_init', create_function('', 'return register_widget("HierNavBreadcrumbs_Widget");'));


### FILE ENDS HERE ###
